"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MempoolSpace = exports.MempoolSpaceClass = void 0;
var request_1 = __importDefault(require("request"));
var util_1 = __importDefault(require("util"));
var logger_1 = __importDefault(require("../../logger"));
var MempoolSpaceClass = /** @class */ (function () {
    function MempoolSpaceClass() {
        // TODO run our own mempool.space server
        this.feeUrls = {
            mainnet: 'https://mempool.space/api/v1/fees/recommended',
            testnet: 'https://mempool.space/testnet/api/v1/fees/recommended'
        };
        this.cache = {
            mainnet: {
                timestamp: 0,
                response: null
            },
            testnet: {
                timestamp: 0,
                response: null
            }
        };
        this.cacheTime = 1000 * 90; // 90 seconds
    }
    MempoolSpaceClass.prototype.getFee = function (network, nblocks) {
        return __awaiter(this, void 0, void 0, function () {
            var res, fee, err_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        network = network === 'regtest' ? 'testnet' : network;
                        if (this.cache[network] && this.cache[network].timestamp > Date.now() - this.cacheTime) {
                            return [2 /*return*/, this._getFeeLevel(this.cache[network].response, nblocks)];
                        }
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, util_1.default.promisify(request_1.default.get).call(request_1.default, {
                                uri: this.feeUrls[network],
                                json: true
                            })];
                    case 2:
                        res = _a.sent();
                        if (res.statusCode !== 200) {
                            if (res.statusCode === 429) {
                                this.cacheTime += 1000 * 30; // add 30 seconds
                            }
                            throw new Error("Status code: ".concat(res.statusCode));
                        }
                        fee = res.body;
                        this.cache[network] = {
                            timestamp: Date.now(),
                            response: fee
                        };
                        return [2 /*return*/, this._getFeeLevel(fee, nblocks)];
                    case 3:
                        err_1 = _a.sent();
                        logger_1.default.warn('Error getting fee from mempool.space: %o', err_1.message || err_1);
                        throw err_1;
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    MempoolSpaceClass.prototype._getFeeLevel = function (response, nblocks) {
        var avgBlockTime = nblocks * 10; // 10 minutes per block on avg
        if (avgBlockTime <= 10) { // 0 or 1 block
            return response.fastestFee;
        }
        else if (avgBlockTime <= 30) {
            return response.halfHourFee;
        }
        else if (avgBlockTime <= 60) {
            return response.hourFee;
        }
        else if (avgBlockTime <= 60 * 24) { // one day
            return response.economyFee;
        }
        else {
            return response.minimumFee;
        }
    };
    return MempoolSpaceClass;
}());
exports.MempoolSpaceClass = MempoolSpaceClass;
;
exports.MempoolSpace = new MempoolSpaceClass();
//# sourceMappingURL=mempoolspace.js.map