"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * parseArgv
 * returns an object from argv
 * @param {Arg[]} required - array of required arguments
 * @param {Arg[]} optional=[] - array of optional arguments to capture
 * @returns {object}
 */
function parseArgv(required, optional) {
    var e_1, _a, _b, _c, e_2, _d, _e, _f;
    if (optional === void 0) { optional = []; }
    var parsed = {};
    try {
        for (var required_1 = __values(required), required_1_1 = required_1.next(); !required_1_1.done; required_1_1 = required_1.next()) {
            var arg = required_1_1.value;
            var type = 'string';
            if (typeof arg === 'object') {
                (_b = arg, arg = _b.arg, type = _b.type);
            }
            var argIndex = process.argv.indexOf("--".concat(arg));
            if (argIndex === -1)
                throw new Error(arg + ' is a required command argument');
            var argValue = convertType(argIndex >= 0 ? process.argv[argIndex + 1] : '', type);
            if (argValue == null || argValue === '')
                throw new Error(arg + ' is missing a value of ' + type + ' type');
            Object.assign(parsed, (_c = {}, _c[arg] = argValue, _c));
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (required_1_1 && !required_1_1.done && (_a = required_1.return)) _a.call(required_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    try {
        for (var optional_1 = __values(optional), optional_1_1 = optional_1.next(); !optional_1_1.done; optional_1_1 = optional_1.next()) {
            var arg = optional_1_1.value;
            var type = 'string';
            if (typeof arg === 'object') {
                (_e = arg, arg = _e.arg, type = _e.type);
            }
            var argIndex = process.argv.indexOf("--".concat(arg));
            var argValue = convertType(argIndex >= 0 ? process.argv[argIndex + 1] : '', type);
            if (argIndex > -1 && (argValue == null || argValue === ''))
                throw new Error(arg + ' is missing a value of ' + type + ' type');
            Object.assign(parsed, (_f = {}, _f[arg] = argValue, _f));
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (optional_1_1 && !optional_1_1.done && (_d = optional_1.return)) _d.call(optional_1);
        }
        finally { if (e_2) throw e_2.error; }
    }
    return parsed;
}
exports.default = parseArgv;
function convertType(val, type) {
    switch (type) {
        case 'string':
        default:
            return val;
        case 'bool':
        case 'boolean':
            val = val === '' ? 'false' : val; // if index of param was -1, then val will be ''
            var givenWithoutVal = val == null || (val === null || val === void 0 ? void 0 : val.startsWith('--'));
            return Boolean(val === 'true' || parseInt(val) || givenWithoutVal);
        case 'int':
            if (!val) {
                return;
            }
            if (isNaN(parseInt(val))) {
                throw new Error("Invalid arg type. Expected int but got \"".concat(val, "\""));
            }
            return parseInt(val);
        case 'number':
        case 'float':
            if (!val) {
                return;
            }
            if (isNaN(parseFloat(val))) {
                throw new Error("Invalid arg type. Expected float but got \"".concat(val, "\""));
            }
            return parseFloat(val);
    }
}
//# sourceMappingURL=parseArgv.js.map