"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var bson_1 = require("bson");
var chai_1 = require("chai");
var sinon_1 = __importDefault(require("sinon"));
var stream_1 = require("stream");
var coin_1 = require("../../../src/models/coin");
var transaction_1 = require("../../../src/models/transaction");
var walletAddress_1 = require("../../../src/models/walletAddress");
var transaction_fixture_1 = require("../../fixtures/transaction.fixture");
var helpers_1 = require("../../helpers");
var unit_1 = require("../../helpers/unit");
var bitcoreLib = require('bitcore-lib');
describe('Transaction Model', function () {
    var _this = this;
    before(unit_1.unitBeforeHelper);
    after(unit_1.unitAfterHelper);
    var sandbox = sinon_1.default.sandbox.create();
    var address = 'mjVf6sFjt9q6aLY7M21Ap6CPSWdaoNHSf1';
    this.timeout(500000);
    before(function () {
        (0, helpers_1.mockStorage)([]);
    });
    afterEach(function () {
        sandbox.restore();
    });
    it('should stream all the mint operations', function () { return __awaiter(_this, void 0, void 0, function () {
        var tx, batches, mintStream, done;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    tx = bitcoreLib.Transaction(transaction_fixture_1.TransactionFixture.transaction);
                    batches = 0;
                    mintStream = new stream_1.Readable({ objectMode: true, read: function () { } });
                    done = new Promise(function (r) {
                        return mintStream
                            .on('data', function (mintOps) {
                            batches++;
                            var ops = mintOps;
                            (0, chai_1.expect)(ops.length).to.eq(1);
                            (0, chai_1.expect)(ops[0].updateOne.update.$set.address).to.eq(address);
                        })
                            .on('end', r);
                    });
                    return [4 /*yield*/, transaction_1.TransactionStorage.streamMintOps({
                            chain: 'BTC',
                            network: 'regtest',
                            txs: [tx],
                            height: 8534,
                            mintStream: mintStream,
                            initialSyncComplete: true
                        })];
                case 1:
                    _a.sent();
                    return [4 /*yield*/, done];
                case 2:
                    _a.sent();
                    (0, chai_1.expect)(batches).to.eq(1);
                    return [2 /*return*/];
            }
        });
    }); });
    it('should batch large amount of transactions', function () { return __awaiter(_this, void 0, void 0, function () {
        var tx, batches, mintStream, done;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    tx = bitcoreLib.Transaction(transaction_fixture_1.TransactionFixture.transaction);
                    batches = 0;
                    mintStream = new stream_1.Readable({ objectMode: true, read: function () { } });
                    done = new Promise(function (r) {
                        return mintStream
                            .on('data', function (mintOps) {
                            batches++;
                            var ops = mintOps;
                            (0, chai_1.expect)(ops.length).to.eq(50000);
                        })
                            .on('end', r);
                    });
                    return [4 /*yield*/, transaction_1.TransactionStorage.streamMintOps({
                            chain: 'BTC',
                            network: 'regtest',
                            txs: new Array(100000).fill(tx),
                            height: 8534,
                            mintStream: mintStream,
                            initialSyncComplete: true
                        })];
                case 1:
                    _a.sent();
                    return [4 /*yield*/, done];
                case 2:
                    _a.sent();
                    (0, chai_1.expect)(batches).to.eq(2);
                    return [2 /*return*/];
            }
        });
    }); });
    it('should stream all the spend operations', function () { return __awaiter(_this, void 0, void 0, function () {
        var tx, batches, CURRENT_HEIGHT, spentStream, done;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    tx = bitcoreLib.Transaction(transaction_fixture_1.TransactionFixture.transaction);
                    batches = 0;
                    CURRENT_HEIGHT = 8534;
                    spentStream = new stream_1.Readable({ objectMode: true, read: function () { } });
                    done = new Promise(function (r) {
                        return spentStream
                            .on('data', function (spentOps) {
                            batches++;
                            var ops = spentOps;
                            (0, chai_1.expect)(ops.length).to.eq(tx.inputs.length);
                            (0, chai_1.expect)(ops[0].updateOne.update.$set.spentHeight).to.eq(CURRENT_HEIGHT);
                            (0, chai_1.expect)(ops[0].updateOne.update.$set.spentTxid).to.eq(tx.hash);
                        })
                            .on('end', r);
                    });
                    return [4 /*yield*/, transaction_1.TransactionStorage.streamSpendOps({
                            chain: 'BTC',
                            network: 'regtest',
                            txs: [tx],
                            height: CURRENT_HEIGHT,
                            spentStream: spentStream
                        })];
                case 1:
                    _a.sent();
                    return [4 /*yield*/, done];
                case 2:
                    _a.sent();
                    (0, chai_1.expect)(batches).to.eq(1);
                    return [2 /*return*/];
            }
        });
    }); });
    describe('Wallet Tagging', function () { return __awaiter(_this, void 0, void 0, function () {
        var tx, CURRENT_HEIGHT, correctWalletId;
        var _this = this;
        return __generator(this, function (_a) {
            tx = bitcoreLib.Transaction(transaction_fixture_1.TransactionFixture.transaction);
            CURRENT_HEIGHT = 8534;
            correctWalletId = new bson_1.ObjectId('5d93abeba811051da3af9a35');
            it('should tag wallets on the mint ops first', function () { return __awaiter(_this, void 0, void 0, function () {
                var mintStream, done;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            sandbox.stub(walletAddress_1.WalletAddressStorage, 'collection').get(function () { return ({
                                find: sandbox.stub().returnsThis(),
                                project: sandbox.stub().returnsThis(),
                                toArray: sandbox.stub().resolves([
                                    { wallet: correctWalletId, address: address },
                                    { wallet: new bson_1.ObjectId('6d93abeba811051da3af9a35'), address: 'fakeaddress' }
                                ])
                            }); });
                            mintStream = new stream_1.Readable({ objectMode: true, read: function () { } });
                            done = new Promise(function (r) {
                                return mintStream
                                    .on('data', function (mintOps) {
                                    var ops = mintOps;
                                    (0, chai_1.expect)(ops.length).to.eq(1);
                                    (0, chai_1.expect)(ops[0].updateOne.update.$set.address).to.eq(address);
                                })
                                    .on('end', r);
                            });
                            return [4 /*yield*/, transaction_1.TransactionStorage.streamMintOps({
                                    chain: 'BTC',
                                    network: 'regtest',
                                    txs: [tx],
                                    height: 8534,
                                    mintStream: mintStream,
                                    initialSyncComplete: true
                                })];
                        case 1:
                            _a.sent();
                            return [4 /*yield*/, done];
                        case 2:
                            _a.sent();
                            (0, chai_1.expect)(tx.wallets).to.exist;
                            (0, chai_1.expect)(tx.wallets.length).to.eq(1);
                            (0, chai_1.expect)(tx.wallets[0]).to.eq(correctWalletId);
                            return [2 /*return*/];
                    }
                });
            }); });
            it('should tag the transaction ops, and calculate the fee', function () { return __awaiter(_this, void 0, void 0, function () {
                function getCoinForInput(i) {
                    var input = i.toObject();
                    var inputTxid = i.toObject().prevTxId;
                    var fixtureInput = transaction_fixture_1.TransactionFixture.inputs[inputTxid];
                    var inputTx = new bitcoreLib.Transaction(fixtureInput);
                    var coin = { spentTxid: tx.hash, value: inputTx.outputs[input.outputIndex].satoshis, wallets: [] };
                    return coin;
                }
                var txStream, done;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            sandbox.stub(coin_1.CoinStorage, 'collection').get(function () { return ({
                                find: sandbox.stub().returnsThis(),
                                project: sandbox.stub().returnsThis(),
                                toArray: sandbox.stub().resolves(tx.inputs.map(getCoinForInput))
                            }); });
                            txStream = new stream_1.Readable({ objectMode: true, read: function () { } });
                            done = new Promise(function (r) {
                                return txStream
                                    .on('data', function (spentOps) {
                                    var ops = spentOps;
                                    (0, chai_1.expect)(ops.length).to.eq(1);
                                    (0, chai_1.expect)(ops[0].updateOne.filter.txid).to.eq(tx.hash);
                                    (0, chai_1.expect)(ops[0].updateOne.update.$set.fee).to.eq(81276);
                                    (0, chai_1.expect)(ops[0].updateOne.update.$set.inputCount).to.eq(tx.inputs.length);
                                    (0, chai_1.expect)(ops[0].updateOne.update.$set.wallets.length).to.eq(1);
                                    (0, chai_1.expect)(ops[0].updateOne.update.$set.wallets[0]).to.eq(correctWalletId);
                                })
                                    .on('end', r);
                            });
                            return [4 /*yield*/, transaction_1.TransactionStorage.streamTxOps({
                                    chain: 'BTC',
                                    network: 'regtest',
                                    txs: [tx],
                                    height: CURRENT_HEIGHT,
                                    initialSyncComplete: false,
                                    txStream: txStream
                                })];
                        case 1:
                            _a.sent();
                            return [4 /*yield*/, done];
                        case 2:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            }); });
            return [2 /*return*/];
        });
    }); });
});
//# sourceMappingURL=transaction.spec.js.map