"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var sinon_1 = __importDefault(require("sinon"));
var modules_1 = require("../../src/modules");
var chain_state_1 = require("../../src/providers/chain-state");
var libs_1 = require("../../src/providers/libs");
var config_1 = require("../../src/services/config");
var p2p_1 = require("../../src/services/p2p");
var verification_1 = require("../../src/services/verification");
var unit_1 = require("../helpers/unit");
describe('Modules', function () {
    before(unit_1.unitBeforeHelper);
    after(unit_1.unitAfterHelper);
    it('should load configured modules correctly', function () {
        var sandbox = sinon_1.default.createSandbox();
        sandbox.stub(config_1.Config, 'get').returns(mockConfig);
        validateModules();
        sandbox.restore();
    });
    it('should prevent double module registration', function () {
        var sandbox = sinon_1.default.createSandbox();
        sandbox.stub(config_1.Config, 'get').returns(__assign({ 
            // Double module registration test case. ModuleManager will attempt to register BTC Module from
            // "config.modules" as well as "config.chains". Should only register once.
            modules: ['./bitcoin'] }, mockConfig));
        validateModules();
        sandbox.restore();
    });
    it('should have a test which runs', function () {
        (0, chai_1.expect)(true).to.equal(true);
    });
    it('should have services registered after loading modules', function () {
        var e_1, _a;
        var chains = config_1.Config.chains();
        try {
            for (var chains_1 = __values(chains), chains_1_1 = chains_1.next(); !chains_1_1.done; chains_1_1 = chains_1.next()) {
                var chain = chains_1_1.value;
                var service = chain_state_1.ChainStateProvider.get({ chain: chain });
                (0, chai_1.expect)(service).to.exist;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (chains_1_1 && !chains_1_1.done && (_a = chains_1.return)) _a.call(chains_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    });
    it('should have libaries registered', function () {
        var e_2, _a;
        var chains = ['BTC', 'BCH'];
        try {
            for (var chains_2 = __values(chains), chains_2_1 = chains_2.next(); !chains_2_1.done; chains_2_1 = chains_2.next()) {
                var chain = chains_2_1.value;
                var service = libs_1.Libs.get(chain);
                (0, chai_1.expect)(service).to.exist;
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (chains_2_1 && !chains_2_1.done && (_a = chains_2.return)) _a.call(chains_2);
            }
            finally { if (e_2) throw e_2.error; }
        }
    });
    it('should have p2p services registered', function () {
        var e_3, _a;
        var chains = ['BTC', 'BCH'];
        try {
            for (var chains_3 = __values(chains), chains_3_1 = chains_3.next(); !chains_3_1.done; chains_3_1 = chains_3.next()) {
                var chain = chains_3_1.value;
                var service = p2p_1.P2P.get(chain);
                (0, chai_1.expect)(service).to.exist;
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (chains_3_1 && !chains_3_1.done && (_a = chains_3.return)) _a.call(chains_3);
            }
            finally { if (e_3) throw e_3.error; }
        }
    });
    it('should have verification services registered', function () {
        var e_4, _a;
        var chains = ['BTC', 'BCH'];
        try {
            for (var chains_4 = __values(chains), chains_4_1 = chains_4.next(); !chains_4_1.done; chains_4_1 = chains_4.next()) {
                var chain = chains_4_1.value;
                var service = verification_1.Verification.get(chain);
                (0, chai_1.expect)(service).to.exist;
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (chains_4_1 && !chains_4_1.done && (_a = chains_4.return)) _a.call(chains_4);
            }
            finally { if (e_4) throw e_4.error; }
        }
    });
});
var mockConfig = {
    chains: {
        BTC: {
            testnet: {
                chainSource: 'p2p',
                trustedPeers: [
                    {
                        host: '127.0.0.1',
                        port: 18333
                    }
                ],
                rpc: {
                    host: '127.0.0.1',
                    port: 18332,
                    username: 'bitpaytest',
                    password: 'local321'
                }
            },
        },
        ETH: {
            dev: {
                trustedPeers: [
                    {
                        host: '127.0.0.1',
                        port: 8545
                    }
                ],
                chainSource: 'p2p',
                provider: {
                    protocol: 'http',
                    host: '127.0.0.1',
                    port: 8545,
                    chain: 'ETH'
                }
            }
        }
    }
};
var validateModules = function () {
    modules_1.Modules.internalServices = []; // Remove all loaded modules from internalServices array for a fresh load
    modules_1.Modules.loadConfigured(); // Re-load modules with stubbed Config.get()
    (0, chai_1.expect)(modules_1.Modules.internalServices.length).to.equal(2);
    (0, chai_1.expect)(modules_1.Modules.internalServices[0].constructor.name).to.equal('BitcoinModule');
    (0, chai_1.expect)(modules_1.Modules.internalServices[1].constructor.name).to.equal('ETHModule');
};
//# sourceMappingURL=modules.spec.js.map