"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var mocha_1 = require("mocha");
var partition_1 = require("../../../src/utils/partition");
var unit_1 = require("../../helpers/unit");
(0, mocha_1.describe)('Partition', function () {
    before(unit_1.unitBeforeHelper);
    after(unit_1.unitAfterHelper);
    it('should split an array of 5 to 5 arrays', function () {
        var testArr = [1, 2, 3, 4, 5];
        var partitioned = (0, partition_1.partition)(testArr, 5);
        (0, chai_1.expect)(partitioned).to.deep.equal([[1, 2, 3, 4, 5]]);
        (0, chai_1.expect)(testArr).to.deep.equal([1, 2, 3, 4, 5]);
    });
    it('should handle 0', function () {
        var testArr = [1, 2, 3, 4, 5];
        var partitioned = (0, partition_1.partition)(testArr, 0);
        (0, chai_1.expect)(partitioned).to.deep.equal([[1], [2], [3], [4], [5]]);
        (0, chai_1.expect)(testArr).to.deep.equal([1, 2, 3, 4, 5]);
    });
    it('should handle one', function () {
        var testArr = [1, 2, 3, 4, 5];
        var partitioned = (0, partition_1.partition)(testArr, 1);
        (0, chai_1.expect)(partitioned).to.deep.equal([[1], [2], [3], [4], [5]]);
        (0, chai_1.expect)(testArr).to.deep.equal([1, 2, 3, 4, 5]);
    });
    it('should handle two', function () {
        var testArr = [1, 2, 3, 4, 5];
        var partitioned = (0, partition_1.partition)(testArr, 2);
        (0, chai_1.expect)(partitioned).to.deep.equal([[1, 2], [3, 4], [5]]);
        (0, chai_1.expect)(testArr).to.deep.equal([1, 2, 3, 4, 5]);
    });
    it('should handle between one and zero', function () {
        var testArr = [1, 2, 3, 4, 5];
        var partitioned = (0, partition_1.partition)(testArr, 0.15);
        (0, chai_1.expect)(partitioned).to.deep.equal([[1], [2], [3], [4], [5]]);
        (0, chai_1.expect)(testArr).to.deep.equal([1, 2, 3, 4, 5]);
    });
    it('should handle different sizes of arrays', function () {
        var _loop_1 = function (i) {
            var randomLen = Math.floor(Math.random() * 100) + 1;
            var adjustment = 0;
            var randomArr = new Array(randomLen).fill(1).map(function (num) { return num + adjustment++; });
            var partitioned = (0, partition_1.partition)(randomArr, 3);
            var amountInBatches = partitioned.reduce(function (sum, arr) { return sum + arr.length; }, 0);
            (0, chai_1.expect)(amountInBatches).to.equal(randomLen);
            (0, chai_1.expect)(partitioned.length).to.be.gte(Math.floor(randomLen / 3));
            var lastBatch = partitioned[partitioned.length - 1];
            if (!lastBatch) {
                console.error('Array partition fails with length', randomLen);
            }
            (0, chai_1.expect)(randomArr.length).to.deep.equal(randomLen);
        };
        for (var i = 0; i < 1000; i++) {
            _loop_1(i);
        }
    });
});
//# sourceMappingURL=partition.unit.js.map