#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transactions = exports.blocks = void 0;
var chai_1 = require("chai");
var config_1 = __importDefault(require("../../src/config"));
var logger_1 = __importDefault(require("../../src/logger"));
var block_1 = require("../../src/models/block");
var coin_1 = require("../../src/models/coin");
var transaction_1 = require("../../src/models/transaction");
var walletAddress_1 = require("../../src/models/walletAddress");
var chain_state_1 = require("../../src/providers/chain-state");
var rpc_1 = require("../../src/rpc");
var storage_1 = require("../../src/services/storage");
var SATOSHI = 100000000.0;
function blocks(info, creds) {
    return __awaiter(this, void 0, void 0, function () {
        var rpc, tip, heights, times, normalizedTimes, cursor, block, truth, coinbase, reward, ours, ourCoinbase, txidset, _loop_1, ours_1, ours_1_1, our, extra, increases;
        var e_1, _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    rpc = new rpc_1.AsyncRPC(creds.username, creds.password, creds.host, creds.port);
                    return [4 /*yield*/, chain_state_1.ChainStateProvider.getLocalTip({ chain: info.chain, network: info.network })];
                case 1:
                    tip = _b.sent();
                    heights = new Array(tip.height).fill(false);
                    times = new Array(tip.height).fill(0);
                    normalizedTimes = new Array(tip.height).fill(0);
                    cursor = block_1.BitcoinBlockStorage.collection.find({
                        chain: info.chain,
                        network: info.network
                    });
                    _b.label = 2;
                case 2: return [4 /*yield*/, cursor.hasNext()];
                case 3:
                    if (!_b.sent()) return [3 /*break*/, 8];
                    return [4 /*yield*/, cursor.next()];
                case 4:
                    block = _b.sent();
                    if (!block)
                        return [3 /*break*/, 8];
                    if (!block.processed)
                        return [3 /*break*/, 2];
                    logger_1.default.info("verifying block ".concat(block.hash, ": ").concat(block.height));
                    // Check there's all unique heights
                    (0, chai_1.expect)(block.height, 'block height').to.be.gte(1);
                    (0, chai_1.expect)(block.height, 'block height').to.be.lte(tip.height);
                    (0, chai_1.expect)(heights[block.height - 1], 'height already used').to.be.false;
                    heights[block.height - 1] = true;
                    // Check times are increasing
                    times[block.height - 1] = block.time.getTime();
                    normalizedTimes[block.height - 1] = block.timeNormalized.getTime();
                    return [4 /*yield*/, rpc.verbose_block(block.hash)];
                case 5:
                    truth = _b.sent();
                    (0, chai_1.expect)(block.height, 'block height').to.equal(truth.height);
                    (0, chai_1.expect)(block.hash, 'block hash').to.equal(truth.hash);
                    (0, chai_1.expect)(block.version, 'block version').to.equal(truth.version);
                    (0, chai_1.expect)(block.merkleRoot, 'block merkle root').to.equal(truth.merkleroot);
                    (0, chai_1.expect)(block.nonce, 'block nonce').to.equal(truth.nonce);
                    (0, chai_1.expect)(block.previousBlockHash, 'block prev hash').to.equal(truth.previousblockhash);
                    (0, chai_1.expect)(block.transactionCount, 'block tx count').to.equal(truth.tx.length);
                    if (info.network !== 'regtest') {
                        (0, chai_1.expect)(block.size, 'block size').to.equal(truth.size);
                    }
                    (0, chai_1.expect)(block.bits.toString(16), 'block bits').to.equal(truth.bits);
                    (0, chai_1.expect)(block.processed, 'block processed').to.equal(true);
                    (0, chai_1.expect)(block.time.getTime(), 'block time').to.equal(truth.time * 1000);
                    if (block.height < tip.height) {
                        (0, chai_1.expect)(block.nextBlockHash, 'block next hash').to.equal(truth.nextblockhash);
                    }
                    coinbase = truth.tx[0];
                    reward = coinbase.vout.reduce(function (a, b) { return a + b.value; }, 0);
                    (0, chai_1.expect)(block.reward, 'block reward').to.equal(Math.round(reward * SATOSHI));
                    return [4 /*yield*/, transaction_1.TransactionStorage.collection
                            .find({
                            chain: info.chain,
                            network: info.network,
                            txid: {
                                $in: truth.tx.map(function (tx) { return tx.txid; })
                            }
                        })
                            .project({
                            txid: true,
                            coinbase: true,
                            blockHash: true,
                            blockHeight: true,
                            blockTime: true,
                            blockTimeNormalized: true
                        })
                            .toArray()];
                case 6:
                    ours = _b.sent();
                    ourCoinbase = ours.filter(function (tx) { return tx.coinbase; });
                    (0, chai_1.expect)(ourCoinbase.length, 'number of coinbases').to.equal(1);
                    (0, chai_1.expect)(ourCoinbase[0].txid, 'coinbase txid to match truth').to.equal(coinbase.txid);
                    txidset = new Set(ours.map(function (tx) { return tx.txid; }));
                    (0, chai_1.expect)(ours.length, 'number of txs').to.equal(truth.tx.length);
                    (0, chai_1.expect)(txidset.size, 'number of unique txs').to.equal(truth.tx.length);
                    _loop_1 = function (our) {
                        // Check every one of our txs is contained in `truth`
                        var tx = truth.tx.find(function (tx) { return tx.txid === our.txid; });
                        (0, chai_1.expect)(tx, 'tx to be in the block').to.not.be.undefined;
                        // Check our txs' block hash matches the mongo block
                        (0, chai_1.expect)(our.blockHash, 'tx block hash').to.equal(block.hash);
                        (0, chai_1.expect)(our.blockHeight, 'tx block height').to.equal(block.height);
                        var time = our.blockTime && our.blockTime.getTime();
                        (0, chai_1.expect)(time, 'tx block time').to.equal(block.time.getTime());
                        var ntime = our.blockTimeNormalized && our.blockTimeNormalized.getTime();
                        (0, chai_1.expect)(ntime, 'tx block time normalized').to.equal(block.timeNormalized.getTime());
                    };
                    try {
                        for (ours_1 = (e_1 = void 0, __values(ours)), ours_1_1 = ours_1.next(); !ours_1_1.done; ours_1_1 = ours_1.next()) {
                            our = ours_1_1.value;
                            _loop_1(our);
                        }
                    }
                    catch (e_1_1) { e_1 = { error: e_1_1 }; }
                    finally {
                        try {
                            if (ours_1_1 && !ours_1_1.done && (_a = ours_1.return)) _a.call(ours_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                    }
                    return [4 /*yield*/, transaction_1.TransactionStorage.collection.countDocuments({
                            chain: info.chain,
                            network: info.network,
                            blockHash: block.hash,
                            txid: {
                                $nin: truth.tx.map(function (tx) { return tx.txid; })
                            }
                        })];
                case 7:
                    extra = _b.sent();
                    (0, chai_1.expect)(extra, 'number of extra transactions').to.equal(0);
                    return [3 /*break*/, 2];
                case 8:
                    // Check the heights are all unique
                    (0, chai_1.expect)(heights.filter(function (h) { return !h; }).length, 'no duplicate heights').to.equal(0);
                    increases = function (l) { return !!l.reduce(function (prev, curr) { return (prev < curr ? curr : undefined); }); };
                    (0, chai_1.expect)(increases(normalizedTimes), 'normalized block times only increase').to.be.true;
                    return [2 /*return*/];
            }
        });
    });
}
exports.blocks = blocks;
function transactions(info, creds) {
    return __awaiter(this, void 0, void 0, function () {
        var rpc, txcursor, tx, truth, ours, ours_2, ours_2_1, our, wallets, e_2_1, ours, nspent, ours_3, ours_3_1, our;
        var e_2, _a, e_3, _b;
        return __generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    rpc = new rpc_1.AsyncRPC(creds.username, creds.password, creds.host, creds.port);
                    txcursor = transaction_1.TransactionStorage.collection.find({
                        chain: info.chain,
                        network: info.network
                    });
                    _c.label = 1;
                case 1: return [4 /*yield*/, txcursor.hasNext()];
                case 2:
                    if (!_c.sent()) return [3 /*break*/, 15];
                    return [4 /*yield*/, txcursor.next()];
                case 3:
                    tx = _c.sent();
                    if (!tx) {
                        return [3 /*break*/, 15];
                    }
                    logger_1.default.info("verifying tx ".concat(tx.txid, ": ").concat(tx.blockHeight));
                    return [4 /*yield*/, rpc.transaction(tx.txid, tx.blockHash)];
                case 4:
                    truth = _c.sent();
                    if (info.network !== 'regtest') {
                        (0, chai_1.expect)(tx.size, 'tx size').to.equal(truth.size);
                    }
                    (0, chai_1.expect)(tx.locktime, 'tx locktime').to.equal(truth.locktime);
                    return [4 /*yield*/, coin_1.CoinStorage.collection
                            .find({
                            network: info.network,
                            chain: info.chain,
                            mintTxid: tx.txid
                        })
                            .toArray()];
                case 5:
                    ours = _c.sent();
                    (0, chai_1.expect)(ours.length, 'number mint txids').to.equal(truth.vout.length);
                    _c.label = 6;
                case 6:
                    _c.trys.push([6, 11, 12, 13]);
                    ours_2 = (e_2 = void 0, __values(ours)), ours_2_1 = ours_2.next();
                    _c.label = 7;
                case 7:
                    if (!!ours_2_1.done) return [3 /*break*/, 10];
                    our = ours_2_1.value;
                    // coins
                    (0, chai_1.expect)(our.mintHeight, 'tx mint height').to.equal(tx.blockHeight);
                    (0, chai_1.expect)(our.value, 'tx mint value').to.equal(Math.round(truth.vout[our.mintIndex].value * SATOSHI));
                    // TODO: why?
                    if (our.address && our.address !== 'false') {
                        (0, chai_1.expect)(truth.vout[our.mintIndex].scriptPubKey.addresses, 'tx mint address').to.include(our.address);
                    }
                    (0, chai_1.expect)(our.coinbase).to.equal(tx.coinbase);
                    // wallets
                    (0, chai_1.expect)(tx.wallets).to.include.members(Array.from(our.wallets));
                    if (!(our.wallets.length > 0)) return [3 /*break*/, 9];
                    return [4 /*yield*/, walletAddress_1.WalletAddressStorage.collection
                            .find({
                            wallet: {
                                $in: our.wallets
                            },
                            address: our.address,
                            chain: info.chain,
                            network: info.network
                        })
                            .toArray()];
                case 8:
                    wallets = _c.sent();
                    (0, chai_1.expect)(wallets.length, 'wallet exists').to.be.greaterThan(0);
                    _c.label = 9;
                case 9:
                    ours_2_1 = ours_2.next();
                    return [3 /*break*/, 7];
                case 10: return [3 /*break*/, 13];
                case 11:
                    e_2_1 = _c.sent();
                    e_2 = { error: e_2_1 };
                    return [3 /*break*/, 13];
                case 12:
                    try {
                        if (ours_2_1 && !ours_2_1.done && (_a = ours_2.return)) _a.call(ours_2);
                    }
                    finally { if (e_2) throw e_2.error; }
                    return [7 /*endfinally*/];
                case 13: return [4 /*yield*/, coin_1.CoinStorage.collection
                        .find({
                        network: info.network,
                        chain: info.chain,
                        spentTxid: tx.txid
                    })
                        .toArray()];
                case 14:
                    ours = _c.sent();
                    nspent = truth.vin.length + (tx.coinbase ? -1 : 0);
                    (0, chai_1.expect)(ours.length, 'number spent txids').to.equal(nspent);
                    try {
                        for (ours_3 = (e_3 = void 0, __values(ours)), ours_3_1 = ours_3.next(); !ours_3_1.done; ours_3_1 = ours_3.next()) {
                            our = ours_3_1.value;
                            (0, chai_1.expect)(our.spentHeight, 'tx spent height').to.equal(tx.blockHeight);
                            (0, chai_1.expect)(tx.wallets).to.include.members(Array.from(our.wallets));
                        }
                    }
                    catch (e_3_1) { e_3 = { error: e_3_1 }; }
                    finally {
                        try {
                            if (ours_3_1 && !ours_3_1.done && (_b = ours_3.return)) _b.call(ours_3);
                        }
                        finally { if (e_3) throw e_3.error; }
                    }
                    return [3 /*break*/, 1];
                case 15: return [2 /*return*/];
            }
        });
    });
}
exports.transactions = transactions;
if (require.main === module)
    (function () { return __awaiter(void 0, void 0, void 0, function () {
        var info, creds;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    info = {
                        chain: process.env.CHAIN || 'BTC',
                        network: process.env.NETWORK || 'testnet'
                    };
                    creds = config_1.default.chains[info.chain][info.network].rpc;
                    return [4 /*yield*/, storage_1.Storage.start({})];
                case 1:
                    _a.sent();
                    logger_1.default.info('verifying blocks');
                    return [4 /*yield*/, blocks(info, creds)];
                case 2:
                    _a.sent();
                    logger_1.default.info('verifying transactions');
                    return [4 /*yield*/, transactions(info, creds)];
                case 3:
                    _a.sent();
                    process.exit();
                    return [2 /*return*/];
            }
        });
    }); })().catch(function (err) {
        logger_1.default.error('%o', err);
        process.exit(1);
    });
//# sourceMappingURL=rpc-verify.js.map